﻿using System;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.ComponentModel;
using System.Collections;
using System.Collections.Generic;

namespace SilverlightApplication75
{
    public class Person : INotifyPropertyChanged, INotifyDataErrorInfo, IEditableObject
    {
        #region INotifyPropertyChanged Members

        public event PropertyChangedEventHandler PropertyChanged;

        private void OnPropertyChanged(string propertyName)
        {
            if (this.PropertyChanged != null)
                this.PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
        }

        #endregion

        private string firstName;
        public string FirstName
        {
            get
            {
                return this.firstName;
            }
            set
            {
                if (this.firstName != value)
                {
                    this.firstName = value;
                    this.OnPropertyChanged("FirstName");
                }
            }
        }

        private string lastName;
        public string LastName
        {
            get
            {
                return this.lastName;
            }
            set
            {
                if (this.lastName != value)
                {
                    this.lastName = value;
                    this.OnPropertyChanged("LastName");
                }
            }
        }

        private int age;
        public int Age
        {
            get
            {
                return this.age;
            }
            set
            {
                if (this.age != value)
                {
                    this.age = value;
                    this.OnPropertyChanged("Age");
                }
            }
        }

        private DateTime birthDay;
        public DateTime BirthDay
        {
            get
            {
                return this.birthDay;
            }
            set
            {
                if (this.birthDay != value)
                {
                    this.birthDay = value;
                    this.OnPropertyChanged("BirthDay");
                }
            }
        }

        #region INotifyDataErrorInfo implementation

        private readonly Dictionary<string, IEnumerable<string>> errors = new Dictionary<string, IEnumerable<string>>();

        public event EventHandler<DataErrorsChangedEventArgs> ErrorsChanged;

        public void SetError(string propertyName, string error)
        {
            IEnumerable<string> currentPropertyError;
            if (this.errors.TryGetValue(propertyName, out currentPropertyError))
            {
                this.errors.Remove(propertyName);
            }

            this.errors.Add(propertyName, new List<string>() { error });
            this.RaiseErrorChanged(propertyName);
        }

        public void ClearErrors()
        {
            this.errors.Clear();
            this.RaiseErrorChanged(string.Empty);
        }

        public void RaiseErrorChanged(string propertyName)
        {
            if (this.ErrorsChanged != null)
            {
                this.ErrorsChanged(this, new DataErrorsChangedEventArgs(propertyName));
            }
        }

        public IEnumerable GetErrors(string propertyName)
        {
            IEnumerable<string> result;
            if (this.errors.TryGetValue(propertyName, out result))
            {
                return result;
            }
            return null;
        }

        public bool HasErrors
        {
            get
            {
                return this.errors.Any();
            }
        }

        #endregion

        #region IEditableObject Members

        public void BeginEdit()
        {
            this.ClearErrors();
        }

        public void CancelEdit()
        {
        }

        public void EndEdit()
        {
            if (string.IsNullOrEmpty(this.FirstName))
            {
                this.SetError("FirstName", "First name is required!");
            }
        }

        #endregion
    }
}
